/*
 * Tue Feb 16 20:10:14 CET 1999
 *
 * DlEngine.java - the WebHunter download engine
 * Copyright (C) 1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;

/*
 * This class is the WebHunter's engine
 */

public class DlEngine
{
  private String contentType;
  private boolean stopped, failed;
  private WebHunter parent;

  /*
   * Constructor, get the parent WebHunter
   */

  DlEngine(WebHunter parent)
  {
    this.parent = parent;
  }

  /*
   * Start the download
   */

  public boolean download(String anUrl, String whereToSave, int repetition, int delay, int bufferSize)
  {
    stopped = false;
    int old_timer, timer, wait;
    for (int i = 0; i < repetition; i++)
    {
      if (readAndSaveBinary(anUrl, whereToSave, bufferSize)) return true;
      if (stopped)
      {
        stopped = false;
        break;
      }
      if (failed) break;
      timer = old_timer = wait = 0;
      while (wait != delay)
      {
        if (stopped) break;
        Date date = new Date();
        SimpleDateFormat df = new SimpleDateFormat("ss");
        timer = Integer.parseInt(df.format(date));
        if (Math.abs(timer - old_timer) == 1)
        {
          wait++;
          parent.setProgressString("Error occured, waiting for retry " + String.valueOf(delay - wait) + "...");
        }
        old_timer = timer;
      }
      if (stopped)
      {
       stopped = false;
       break;
      }
    }
    return false;
  }

  /*
   * Is download active or not ?
   */

  public boolean getStopped()
  {
    return stopped;
  }

  /*
   * Show an error message
   */

  public void showErrorMessage(String msg, String title, boolean clear)
  {
    if (clear) parent.setUrlVoid();
    parent.showMessage(msg, title);
  }

  /*
   * Stop download
   */

  public void stopDownload()
  {
    stopped = true;
  }

  /*
   * The engine itself
   */

  private boolean readAndSaveBinary(String urlName, String whereToSave, int bufferSize)
  {
    int nbBytes = 0;
    int length = -1;
    int oldPercent = 0;
    int newPercent;
    try
    {
      parent.setProgressString("Connecting...");
      URL url = new URL(urlName);
      URLConnection urlc = url.openConnection();
      if (stopped) return false;
      length = urlc.getContentLength();
      if (length == -1)
      {
        showErrorMessage("File to hunt does not exist !", "Connect Error", true);
        failed = true;
        return false;
      }
      contentType = urlc.getContentType();
      parent.setContentIcon(contentType, urlName);
      if (stopped) return false;

      DataInputStream in = new DataInputStream(url.openStream());
      if (whereToSave.indexOf(".WebHunter") == -1) whereToSave += ".WebHunter";
      RandomAccessFile out = new RandomAccessFile(whereToSave, "rw");
      int off = 0;
      byte buffer[] = new byte[bufferSize * 1024];
      if ((out.length() < length) && (out.length() > 0))
      {
        in.skipBytes((int) out.length());
        out.seek(out.length());
        nbBytes = (int) out.length();
      }
      for (;;)
      {
        if (stopped)
        {
          in.close();
          out.close();
          return false;
        }
        if ((off = in.read(buffer, 0, bufferSize * 1024)) == -1) break;
        out.write(buffer, 0, off);
        nbBytes += off;
        newPercent = (int) (((float) nbBytes / (float) length) * 100);
        if (oldPercent != newPercent)
        {
          parent.setProgressValue(newPercent, nbBytes, length);
          oldPercent = newPercent;
        }
      }
      in.close();
      out.close();
    } catch (Exception e) {
      return false;
    }
    if (nbBytes != length)
    {
      showErrorMessage("Downloaded file is corrupt !", "Download Error", false);
      failed = true;
      return false;
    }
    return true;
  }
}

// End of DlEngine.java
