/*
 * Mon Feb 15 18:57:27 CET 1999
 *
 * HTMLViewer.java - A mini HTML viewer
 * Copyright (C) 1999 Romain Guy - thanks to Slava Pestov
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.*;
import java.net.*;
import javax.swing.*;
import java.awt.event.*;
import java.io.IOException;
import javax.swing.event.*;
import javax.swing.text.html.*;

public class HTMLViewer extends JFrame implements ActionListener, WindowListener, HyperlinkListener
{
  private JButton back, forward, home, close, go, neww;
  private JTextField site;
  private JEditorPane viewer;
  private URL[] history;
  private int historyPos;

  /*
   * Constructor
   */

  public HTMLViewer(URL url)
  {
    super("JCrawler - The mini Java browser - WebHunter");

    history = new URL[25];

    getContentPane().setLayout(new BorderLayout());

    JPanel buttons = new JPanel();
    back = new JButton("Back");
    back.setMnemonic('B');
    back.setToolTipText("Get back to previous visited page");
    back.addActionListener(this);
    buttons.add(back);
    forward = new JButton("Forward");
    forward.setMnemonic('F');
    forward.setToolTipText("Go to next visited page");
    forward.addActionListener(this);
    buttons.add(forward);
    home = new JButton("Home");
    home.setMnemonic('H');
    home.setToolTipText("Go back home");
    home.addActionListener(this);
    buttons.add(home);
    close = new JButton("Close");
    close.setMnemonic('C');
    close.setToolTipText("Close and quit JCrawler");
    close.addActionListener(this);
    buttons.add(close);
    JPanel urlPane = new JPanel();
    neww = new JButton("New");
    neww.setMnemonic('N');
    neww.setToolTipText("Open a new window");
    neww.addActionListener(this);
    urlPane.add(neww);
    site = new JTextField(24);
    site.setToolTipText("Please input an URL here");
    urlPane.add(site);
    go = new JButton("Go");
    go.setMnemonic('G');
    go.setToolTipText("Go to specified URL");
    go.addActionListener(this);
    urlPane.add(go);

    getContentPane().add("North", buttons);
    getContentPane().add("South", urlPane);

    viewer = new JEditorPane();
    viewer.setEditable(false);
    viewer.addHyperlinkListener(this);
    getContentPane().add("Center", new JScrollPane(viewer));

    if (url != null)
      gotoURL(url, true, true);
    else
      gotoURL(getClass().getResource("html/blank.html"), false, false);

    setSize(600, 400);

    Dimension screen = getToolkit().getScreenSize();
    setLocation((screen.width - getSize().width) / 2, (screen.height - getSize().height) / 2);
    show();
  }

  /*
   * Action Listener
   */

  public void actionPerformed(ActionEvent evt)
  {
    Object source = evt.getSource();
    if (source == back)
    {
      if (historyPos <= 1)
        getToolkit().beep();
      else
      {
        URL url = history[--historyPos - 1];
        gotoURL(url, false, true);
      }
    }
    else if (source == forward)
    {
      if (history.length - historyPos <= 1)
        getToolkit().beep();
      else
      {
        URL url = history[historyPos];
        if (url == null)
          getToolkit().beep();
        else
        {
          historyPos++;
          gotoURL(url, false, true);
        }
      }
    }
    else if (source == home)
      gotoURL(getClass().getResource("html/blank.html"), false, false);
    else if (source == close)
      dispose();
    else if (source == neww)
      new HTMLViewer(null);
    else if (source == go)
    {
      try
      {
        if (site.getText() != null) gotoURL(new URL(site.getText()), true, true);
      } catch (MalformedURLException mue) {
        gotoURL(getClass().getResource("html/error.html"), false, false);
      }
    }
  }

  /*
   * Hyperlinks listener
   */

  public void hyperlinkUpdate(HyperlinkEvent evt)
  {
    if (evt.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
    {
      if (evt instanceof HTMLFrameHyperlinkEvent)
      {
        ((HTMLDocument) viewer.getDocument()).processHTMLFrameHyperlinkEvent((HTMLFrameHyperlinkEvent) evt);
      }
      else
      {
        gotoURL(evt.getURL(), true, true);
      }
    }
  }

  /*
   * Proceed to page change
   */

  public void gotoURL(URL url, boolean addToHistory, boolean show)
  {
    try
    {
      viewer.setPage(url);
      if (show)
        setTitle(url.toString());
      else
        setTitle("JCrawler - The mini Java browser - WebHunter");
      if (addToHistory)
      {
        history[historyPos++] = url;
        if(history.length == historyPos)
          System.arraycopy(history, 1, history, 0, history.length);
        history[historyPos] = null;
      }
    }
    catch (IOException io)
    {
      Object[] args = { io.getMessage() };
      System.err.println(args);
    }
  }

  /*
   * Abstracts methods
   */

  public void windowClosing(WindowEvent evt)
  {
    dispose();
  }
  public void windowOpened(WindowEvent evt) { }
  public void windowClosed(WindowEvent evt) { }
  public void windowIconified(WindowEvent evt) { }
  public void windowDeiconified(WindowEvent evt) { }
  public void windowActivated(WindowEvent evt) { }
  public void windowDeactivated(WindowEvent evt) { }
}

// End of HTMLViewer.java
