/*
 * Tue Feb 16 20:09:11 CET 1999
 *
 * OptionsDialog.java - Options dialog
 * Copyright (C) 1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.Properties;

public class OptionsDialog extends JDialog implements ActionListener, KeyListener, WindowListener
{
  /*
   * Easy GridBagLayout building
   */

  void buildConstraints(GridBagConstraints agbc, int agx, int agy, int agw, int agh, int awx, int awy)
  {
    agbc.gridx = agx;
    agbc.gridy = agy;
    agbc.gridwidth = agw;
    agbc.gridheight = agh;
    agbc.weightx = awx;
    agbc.weighty = awy;
    agbc.insets = new Insets(2, 2, 2, 2);
  }

  /*
   * Private variables/objects
   */

  private String defaultPath;
  private Properties props;
  private WebHunter parent;
  private JSlider slideDelay, slideTries, slideBuffer;
  private JButton btnOk, btnCancel, btnSave;
  private JLabel lblPath, lblTries, lblDelay, lblNote, lblBuffer;

  /*
   * Constructor, get the parent WebHunter
   */

  OptionsDialog(WebHunter parent, int delayValue, int triesValue, int bufferSize, String defaultPath)
  {
    super(parent, "Download options", true);
    this.parent = parent;
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints constraints = new GridBagConstraints();
    getContentPane().setLayout(gridbag);
    this.defaultPath = defaultPath;

    lblPath = new JLabel("Path: ");
    lblPath.setToolTipText(defaultPath);
    btnSave = new JButton("Choose Default");
    btnSave.setToolTipText("Choose a default path for saving");
    btnSave.setMnemonic('C');
    slideDelay = new JSlider(0, 60, 10);
    slideDelay.setPaintTicks(true);
    slideDelay.setMinorTickSpacing(1);
    slideDelay.setMajorTickSpacing(10);
    slideDelay.setPaintLabels(true);
    slideDelay.setSnapToTicks(true);
    slideDelay.setValue(delayValue);
    lblDelay = new JLabel("Delay:");
    slideTries = new JSlider(1, 9, 5);
    slideTries.setPaintTicks(true);
    slideTries.setMinorTickSpacing(1);
    slideTries.setMajorTickSpacing(4);
    slideTries.setPaintLabels(true);
    slideTries.setSnapToTicks(true);
    slideTries.setValue(triesValue);
    lblTries = new JLabel("Tries:");
    slideBuffer = new JSlider(0, 32, 1);
    slideBuffer.setPaintTicks(true);
    slideBuffer.setMinorTickSpacing(1);
    slideBuffer.setMajorTickSpacing(4);
    slideBuffer.setPaintLabels(true);
    slideBuffer.setSnapToTicks(true);
    slideBuffer.setValue(bufferSize);
    lblBuffer = new JLabel("Buffer:");
    lblNote = new JLabel("Note: options will immediately take effect");
    btnOk = new JButton("Ok");
    btnOk.setMnemonic('O');
    btnOk.setToolTipText("Save options");
    btnCancel = new JButton("Cancel");
    btnCancel.setMnemonic('C');
    btnCancel.setToolTipText("Quit without saving");

    buildConstraints(constraints, 0, 0, 1, 1, 50, 50);
    constraints.fill = GridBagConstraints.NONE;
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(lblPath, constraints);
    getContentPane().add(lblPath);

    buildConstraints(constraints, 1, 0, 1, 1, 50, 50);
    constraints.fill = GridBagConstraints.NONE;
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(btnSave, constraints);
    getContentPane().add(btnSave);

    buildConstraints(constraints, 0, 1, 1, 1, 25, 50);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(lblDelay, constraints);
    getContentPane().add(lblDelay);

    buildConstraints(constraints, 1, 1, 1, 1, 75, 50);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(slideDelay, constraints);
    getContentPane().add(slideDelay);

    buildConstraints(constraints, 0, 2, 1, 1, 25, 50);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(lblTries, constraints);
    getContentPane().add(lblTries);

    buildConstraints(constraints, 1, 2, 1, 1, 75, 50);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(slideTries, constraints);
    getContentPane().add(slideTries);

    buildConstraints(constraints, 0, 3, 1, 1, 25, 50);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(lblBuffer, constraints);
    getContentPane().add(lblBuffer);

    buildConstraints(constraints, 1, 3, 1, 1, 75, 50);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(slideBuffer, constraints);
    getContentPane().add(slideBuffer);

    buildConstraints(constraints, 0, 4, 1, 1, 50, 50);
    constraints.fill = GridBagConstraints.NONE;
    constraints.anchor = GridBagConstraints.EAST;
    gridbag.setConstraints(btnOk, constraints);
    getContentPane().add(btnOk);

    buildConstraints(constraints, 1, 4, 1, 1, 50, 50);
    constraints.fill = GridBagConstraints.NONE;
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(btnCancel, constraints);
    getContentPane().add(btnCancel);

    buildConstraints(constraints, 0, 5, 2, 1, 100, 100);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(lblNote, constraints);
    getContentPane().add(lblNote);

    btnOk.addActionListener(this);
    btnSave.addActionListener(this);
    btnCancel.addActionListener(this);

    addKeyListener(this);
    addWindowListener(this);
    Dimension screen = getToolkit().getScreenSize();
    pack();
    setLocation((screen.width - getSize().width) / 2, (screen.height - getSize().height) / 2);
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    show();
  }

  /*
   * Save properties
   */

  public void ok()
  {
    parent.setProperty("retry.many", String.valueOf(slideTries.getValue()));
    parent.setProperty("retry.delay", String.valueOf(slideDelay.getValue()));
    parent.setProperty("buffer.size", String.valueOf(slideBuffer.getValue() == 0 ? slideBuffer.getValue() + 1 : slideBuffer.getValue() ));
    parent.setProperty("default.path", defaultPath);
    parent.saveProps();
    parent.loadProperties();
    dispose();
  }

  /*
   * Choose a default save path
   */

  public void choosePath()
  {
    JFileChooser chooser = new JFileChooser(defaultPath);
    chooser.setDialogType(JFileChooser.OPEN_DIALOG);
    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    int retVal = chooser.showDialog(this, null);
    if (retVal == JFileChooser.APPROVE_OPTION)
    {
      defaultPath = chooser.getSelectedFile().getAbsolutePath();
      lblPath.setToolTipText(defaultPath);
    }
  }

  /*
   * ActionListener
   */

  public void actionPerformed(ActionEvent evt)
  {
    Object source = evt.getSource();
    if (source == btnOk)
      ok();
    else if (source == btnCancel)
      dispose();
    else if (source == btnSave)
      choosePath();
  }

  /*
   * KeyListener
   */

  public void keyPressed(KeyEvent evt)
  {
    switch(evt.getKeyCode())
    {
      case KeyEvent.VK_ENTER:
        ok();
        break;
    }
  }

  /*
   * Abstract methods
   */

  public void keyReleased(KeyEvent evt) { }
  public void keyTyped(KeyEvent evt) { }

  public void windowClosing(WindowEvent evt)
  {
    dispose();
  }
  public void windowOpened(WindowEvent evt) { }
  public void windowClosed(WindowEvent evt) { }
  public void windowIconified(WindowEvent evt) { }
  public void windowDeiconified(WindowEvent evt) { }
  public void windowActivated(WindowEvent evt) { }
  public void windowDeactivated(WindowEvent evt) { }
}

// End of OptionsDialog.java
