/*
 * Sun Feb 28 14:50:59 CET 1999
 *
 * WebHunter.java - WebHunter's GUI
 * Copyright (C) 1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.awt.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;

/*
 * The WebHunter class implements Runnable (threads) and some listeners
 */

public class WebHunter extends JFrame implements Runnable, ActionListener, WindowListener
{
  public static final String USER_PROPS = System.getProperty("user.home") + File.separator + ".webhunter.props";
  public static final String BUILD = new String("19990228");
  public static final String VERSION = new String("1.2");

  private final int NO_SESSION = 0;
  private final int SAVE_SESSION = 1;
  private final int LOAD_SESSION = 2;

  private static int tryDelay, manyTries, bufferSize;
  private static Vector instances = new Vector();
  private static String defaultPath, usrProps;
  private static Properties props;

  private JButton btnStart, btnStop, btnOption, btnWeb, btnSave, btnSaveSession, btnLoadSession, btnInstance;
  private JLabel lUrlName, lStorage, lContent, lStatus;
  private JTextField tUrl, tStorage;
  private JProgressBar current;
  private DlEngine downloader;
  private boolean stopped;
  private String fileName;
  private Thread thread;

  /*
   * Useful for automazing GridBagLayout construction
   */

  void buildConstraints(GridBagConstraints agbc, int agx, int agy, int agw, int agh, int awx, int awy)
  {
    agbc.gridx = agx;
    agbc.gridy = agy;
    agbc.gridwidth = agw;
    agbc.gridheight = agh;
    agbc.weightx = awx;
    agbc.weighty = awy;
    agbc.insets = new Insets(2, 2, 2, 2);
  }

  /*
   * The Graphical User Interface
   */

  public WebHunter(String title)
  {
    super(title);
    thread = null;
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints constraints = new GridBagConstraints();
    getContentPane().setLayout(gridbag);

    /*
     * Creating buttons
     */

    btnStart = new JButton("Start Download");
    btnStart.setEnabled(false);
    btnStart.setToolTipText("Start/Resume the URL download");
    btnStart.setMnemonic('D');

    btnStop = new JButton("Stop Download");
    btnStop.setEnabled(false);
    btnStop.setToolTipText("Stop download for resume later");
    btnStop.setMnemonic('T');

    btnSaveSession = new JButton("Save Session");
    btnSaveSession.setEnabled(false);
    btnSaveSession.setToolTipText("Save current download session");
    btnSaveSession.setMnemonic('E');

    btnLoadSession = new JButton("Load Session");
    btnLoadSession.setToolTipText("Open a previous download session");
    btnLoadSession.setMnemonic('L');

    btnInstance = new JButton("New Window");
    btnInstance.setToolTipText("Open a new WebHunter window");
    btnInstance.setMnemonic('W');

    btnWeb = new JButton("Web");
    btnWeb.setToolTipText("Open the WebHunter's mini browser");
    btnWeb.setMnemonic('B');

    btnOption = new JButton("Options");
    btnOption.setToolTipText("Show options");
    btnOption.setMnemonic('O');

    btnSave = new JButton("Save as...");
    btnSave.setDefaultCapable(true);
    btnSave.setToolTipText("Select a place where to save the download");
    btnSave.setMnemonic('S');

    /*
     * Creating text fields and porgress bar
     */

    tUrl = new JTextField("");
    tUrl.setCursor(new Cursor(Cursor.TEXT_CURSOR));
    tStorage = new JTextField("");
    tStorage.setEditable(false);
    lUrlName = new JLabel("        File to hunt: ");
    lStorage = new JLabel("        Save as: ");
    lContent = new JLabel(new ImageIcon(getClass().getResource("resource/title.jpg")));
    lContent.setToolTipText("Download type");
    lStatus = new JLabel("(C)1999 Romain Guy - www.chez.com/powerteam");
    lStatus.setToolTipText("Status Bar");
    current = new JProgressBar(JScrollBar.HORIZONTAL, 0, 100);
    current.setStringPainted(true);
    current.setBorderPainted(true);

    /*
     * Setting up listeners
     */

    btnSave.addActionListener(this);
    btnStop.addActionListener(this);
    btnWeb.addActionListener(this);
    btnStart.addActionListener(this);
    btnOption.addActionListener(this);
    btnInstance.addActionListener(this);
    btnLoadSession.addActionListener(this);
    btnSaveSession.addActionListener(this);
    addWindowListener(this);

    /*
     * Building layout
     */

    buildConstraints(constraints, 0, 0, 1, 1, 25, 25);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(btnSave, constraints);
    getContentPane().add(btnSave);

    buildConstraints(constraints, 1, 0, 1, 1, 25, 25);
    constraints.fill = GridBagConstraints.NONE;
    gridbag.setConstraints(btnStart, constraints);
    getContentPane().add(btnStart);

    buildConstraints(constraints, 2, 0, 1, 1, 25, 25);
    constraints.fill = GridBagConstraints.NONE;
    gridbag.setConstraints(btnStop, constraints);
    getContentPane().add(btnStop);

    buildConstraints(constraints, 3, 0, 1, 2, 25, 25);
    constraints.fill = GridBagConstraints.NONE;
    gridbag.setConstraints(lContent, constraints);
    getContentPane().add(lContent);

    buildConstraints(constraints, 0, 1, 1, 1, 50, 50);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(btnSaveSession, constraints);
    getContentPane().add(btnSaveSession);

    buildConstraints(constraints, 1, 1, 1, 1, 25, 25);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    gridbag.setConstraints(btnLoadSession, constraints);
    getContentPane().add(btnLoadSession);

    buildConstraints(constraints, 2, 1, 1, 1, 25, 25);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    gridbag.setConstraints(btnOption, constraints);
    getContentPane().add(btnOption);

    buildConstraints(constraints, 0, 2, 1, 1, 25, 25);
    constraints.fill = GridBagConstraints.NONE;
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(lUrlName, constraints);
    getContentPane().add(lUrlName);

    buildConstraints(constraints, 1, 2, 3, 1, 75, 25);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    gridbag.setConstraints(tUrl, constraints);
    getContentPane().add(tUrl);

    buildConstraints(constraints, 0, 3, 1, 1, 25, 25);
    constraints.fill = GridBagConstraints.NONE;
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(lStorage, constraints);
    getContentPane().add(lStorage);

    buildConstraints(constraints, 1, 3, 3, 1, 75, 25);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    gridbag.setConstraints(tStorage, constraints);
    getContentPane().add(tStorage);

    buildConstraints(constraints, 0, 4, 2, 1, 50, 50);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(current, constraints);
    getContentPane().add(current);

    buildConstraints(constraints, 2, 4, 1, 1, 25, 25);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(btnInstance, constraints);
    getContentPane().add(btnInstance);

    buildConstraints(constraints, 3, 4, 1, 1, 25, 25);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    gridbag.setConstraints(btnWeb, constraints);
    getContentPane().add(btnWeb);

    buildConstraints(constraints, 0, 5, 4, 1, 100, 100);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    gridbag.setConstraints(lStatus, constraints);
    getContentPane().add(lStatus);

    /*
     * Misceallenous
     */

    pack();
    Dimension screen = getToolkit().getScreenSize();
    setLocation((screen.width - getSize().width) / 2, (screen.height - getSize().height) / 2);
  }

  /*
   * Actions management
   */

  public void actionPerformed(ActionEvent e)
  {
    JComponent source = (JComponent) e.getSource();
    if (source == btnWeb)
      new HTMLViewer(null);
    else if (source == btnInstance)
      addInstance(true);
    else if (source == btnStop)
      stopDownload();
    else if (source == btnStart)
      startDownload();
    else if (source == btnSave)
      saveFile();
    else if (source == btnOption)
      new OptionsDialog(this, tryDelay, manyTries, bufferSize, defaultPath);
    else if (source == btnSaveSession) {
      if (!saveSession()) showMessage("Error has occured while saving the session !", "Session Error");
    } else if (source == btnLoadSession) {
      if (!loadSession()) showMessage("Error has occured while loading the session !", "Session Error");
    }
  }

  public void windowClosing(WindowEvent e)
  {
    quitWebHunter();
  }

  /*
   * Abstract methods
   */

  public void windowClosed(WindowEvent evt) { }
  public void windowIconified(WindowEvent evt) { }
  public void windowOpened(WindowEvent evt) { }
  public void windowDeiconified(WindowEvent evt) { }
  public void windowActivated(WindowEvent evt) { }
  public void windowDeactivated(WindowEvent evt) { }

  /*
   * Sarting download
   */

  public void run()
  {
    downloader = new DlEngine(this);
    boolean result = downloader.download(tUrl.getText(), tStorage.getText(), manyTries, tryDelay, bufferSize);
    btnSave.setEnabled(true);
    btnStop.setEnabled(false);
    btnOption.setEnabled(true);
    btnLoadSession.setEnabled(true);
    tUrl.setEnabled(true);
    if (result)
    {
      String data = new String(tStorage.getText());
      if (data.indexOf(".WebHunter") == -1) data += ".WebHunter";
      File source = new File(data);
      if (!source.exists()) showMessage("Cannot find downloaded file !", "File Error");
      if (!source.renameTo(new File(tStorage.getText()))) showMessage("Cannot rename file !", "File Error");
      btnStart.setEnabled(false);
      tStorage.setText("");
      setUrlVoid();
      btnSaveSession.setEnabled(false);
      lContent.setIcon(new ImageIcon(getClass().getResource("resource/title.jpg")));
      setProgressValue(0, 0, 0);
      setProgressString("Download complete");
    } else {
      if (stopped)
      {
        setProgressString("Download stopped");
        stopped = false;
      } else {
        setProgressValue(0, 0, 0);
        setProgressString("Download failed");
      }
      btnStart.setEnabled(true);
      btnSaveSession.setEnabled(true);
    }
    this.setTitle("WebHunter");
    thread = null;
  }

  /*
   * Buttons 'listeners'
   */

  public void quitWebHunter()
  {
    if (!stopped) stopDownload();
    if (getElements() == 1)
      System.exit(0);
    else
      removeInstance();
  }

  public void stopDownload()
  {
    if (thread != null)
    {
      downloader.stopDownload();
      stopped = true;
    }
  }

  public void saveFile()
  {
    String save = choosePlace(true, NO_SESSION, "Save as");
    if (save != null)
    {
      if (new File(save).exists())
      {
        if (JOptionPane.showConfirmDialog(null, "File already exists. Overwrite it ?", "Save Query", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == 0)
        {
          new File(save).delete();
          tStorage.setText(save);
          btnStart.setEnabled(true);
          btnSaveSession.setEnabled(true);
        }
      } else {
        tStorage.setText(save);
        btnStart.setEnabled(true);
        btnSaveSession.setEnabled(true);
      }
    }
  }

  public void startDownload()
  {
    if (!tUrl.getText().equals(null))
    {
      if (!checkURL())
      {
        showMessage("This is not a valid URL !", "Connect Error");
        setUrlVoid();
      }
      else {
        btnStart.setEnabled(false);
        btnSave.setEnabled(false);
        btnOption.setEnabled(false);
        btnStop.setEnabled(true);
        btnLoadSession.setEnabled(false);
        tUrl.setEnabled(false);
        fileName = getFileName(tUrl.getText());
        repaint();
        if (thread == null)
        {
          thread = new Thread(this);
          thread.start();
        }
      }
    } else
      showMessage("Please specify an URL !", "Connect Error");
  }

  public boolean loadSession()
  {
    try
    {
      String data = choosePlace(false, LOAD_SESSION, "Load a session");
      if (data != null)
      {
        RandomAccessFile session = new RandomAccessFile(data, "r");
        if (!session.readLine().equals("[WebHunter Session File]")) return false;
        tUrl.setText(session.readLine());
        tStorage.setText(session.readLine());
        session.close();
        btnStart.setEnabled(true);
      }
    } catch (FileNotFoundException fnf) {
      return false;
    } catch (IOException ioe) { return false; }
    return true;
  }

  public boolean saveSession()
  {
    try
    {
      if (tUrl.getText() == null)
        return false;
      else if (!checkURL()) {
        showMessage("This is not a valid URL !", "Connect Error");
        setUrlVoid();
        return false;
      }
      String data = choosePlace(true, SAVE_SESSION, "Save a session");
      if (data.indexOf(".whsession") == -1) data += ".whsession";
      if (data != null)
      {
        if (new File(data).exists())
        {
          if (JOptionPane.showConfirmDialog(null, "File already exists. Overwrite it ?", "Save Query", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == 0)
          {
            new File(data).delete();
            RandomAccessFile session = new RandomAccessFile(data, "rw");
            session.writeBytes("[WebHunter Session File]");
            session.write(Character.LINE_SEPARATOR);
            session.writeBytes(tUrl.getText());
            session.write(Character.LINE_SEPARATOR);
            session.writeBytes(tStorage.getText());
            session.close();
          }
        } else {
          RandomAccessFile session = new RandomAccessFile(data, "rw");
          session.writeBytes("[WebHunter Session File]");
          session.write(Character.LINE_SEPARATOR);
          session.writeBytes(tUrl.getText());
          session.write(Character.LINE_SEPARATOR);
          session.writeBytes(tStorage.getText());
          session.close();
        }
      }
    } catch (FileNotFoundException fnf) {
      return false;
    } catch (IOException ioe) { return false; }
    return true;
  }

  /*
   * Change icon according to the content type (see DlEngine)
   */

  public void setContentIcon(String contentType, String fileName)
  {
    String extension = fileName.substring(fileName.lastIndexOf(".") + 1);
    String icon = new String("resource/file-dot-");
    String ext = new String(".jpg");
    if (contentType.equals("application/octet-stream"))
    {
      if (!(extension.equals("exe")) || !(extension.equals("gz")))
        extension = "exe";
    } else if (contentType.equals("text/html")) {
      if (JOptionPane.showConfirmDialog(null, "Would you like to see this HTML page ?", "HTML Query", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == 0)
      {
        try
        {
          new HTMLViewer(new URL(fileName));
        } catch (MalformedURLException mue) {
          new HTMLViewer(null);
        }
      }
      extension = "htm";
    } else if (contentType.equals("image/gif"))
      extension = "gif";
    else if (contentType.equals("image/jpeg"))
      extension = "jpg";
    else if (contentType.equals("application/x-troff-msvideo"))
      extension = "avi";
    else if (contentType.equals("text/plain")) {
      if (!(extension.equals("txt")) && !(extension.equals("java")) && !(extension.equals("c")) && !(extension.equals("cc")))
        extension = "txt";
    } else if (contentType.equals("content/unknown")) {
      if (!(extension.equals("jar")) && !(extension.equals("class")) && !(extension.equals("mp3")) && !(extension.equals("tga")) && !(extension.equals("tgz")))
        extension = "plain";
    } else if (contentType.equals("application/x-tar"))
      extension = "tar";
    else if (contentType.equals("application/x-wav"))
      extension = "wav";
    else if (contentType.equals("application/x-zip"))
      extension = "zip";
    else if (contentType.equals("application/x-bitmap"))
      extension = "xpm";
    lContent.setIcon(new ImageIcon(getClass().getResource(icon + extension + ext)));
  }

  /*
   * Properties management
   */

  // ---- Load the property file ----

  public static void loadProps(InputStream in, String name) throws FileNotFoundException, IOException
  {
    props = new Properties();
    if (in == null)
      in = new FileInputStream(name);
    props.load(in);
    in.close();
  }

  // ---- Load all properties ----

  public static void loadProperties()
  {
    try
    {
      bufferSize = Integer.parseInt(getProperty("buffer.size"));
    } catch (NumberFormatException nf) { bufferSize = 1; }
    try
    {
      manyTries = Integer.parseInt(getProperty("retry.many"));
    } catch (NumberFormatException nf) { manyTries = 5; }
    try
    {
      tryDelay = Integer.parseInt(getProperty("retry.delay"));
    } catch (NumberFormatException nf) { tryDelay = 10000; }
    defaultPath = getProperty("default.path");
  }

  // ---- Save properties ----

  public void saveProps()
  {
    if(usrProps != null)
    {
      try
      {
        OutputStream out = new FileOutputStream(usrProps);
        props.store(out, "WebHunter Properties");
        out.close();
      }
      catch (IOException io)
      {
        io.printStackTrace();
      }
    }
  }

  // ---- Set a property ----

  public static void setProperty(String name, String value)
  {
    props.put(name, value);
  }

  // ---- Get a property ----

  public static String getProperty(String name)
  {
    return props.getProperty(name);
  }

  /*
   * Get the name of a file from its entire path
   */

  public String getFileName(String path)
  {
    int indx = path.lastIndexOf("/");
    if (indx == -1) return null;
    return path.substring(indx + 1);
  }

  /*
   * Return the name of the file to be saved
   */

  public String choosePlace(boolean url, int session, String title)
  {
    String type = null;
    JFileChooser chooser = new JFileChooser(defaultPath);
    if ((url) && (checkURL())) chooser.setSelectedFile(new File(getFileName(tUrl.getText())));
    chooser.setDialogTitle(title);
    chooser.setDialogType(JFileChooser.SAVE_DIALOG);
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    chooser.setFileHidingEnabled(true);
    if (session == LOAD_SESSION)
    {
      WHFileFilter filter = new WHFileFilter("whsession", "WebHunter Session File");
      chooser.addChoosableFileFilter(filter);
      chooser.setFileFilter(filter);
      type = "Load";
      chooser.setApproveButtonMnemonic('L');
      chooser.setApproveButtonToolTipText("Open selected session");
    } else if (session == SAVE_SESSION) {
      WHFileFilter filter = new WHFileFilter("whsession", "WebHunter Session File");
      chooser.addChoosableFileFilter(filter);
      chooser.setFileFilter(filter);
      type = "Save";
      chooser.setApproveButtonMnemonic('S');
      chooser.setApproveButtonToolTipText("Save the session");
    }
    int retVal = chooser.showDialog(this, type);
    if (retVal == JFileChooser.APPROVE_OPTION) return chooser.getSelectedFile().getAbsolutePath();
    return null;
  }

  /*
   * Check if the URL is valid
   */

  public boolean checkURL()
  {
    if ((!(tUrl.getText().startsWith("file:///")) && !(tUrl.getText().startsWith("ftp://")) && !(tUrl.getText().startsWith("http://"))) || (getFileName(tUrl.getText()) == null))
      return false;
    else
      return true;
  }

  /*
   * Indicators management
   */

  // ---- Set the percent value of the progress bar and display the total bytes downloaded ----

  public void setProgressValue(int percent, int csize, int tsize)
  {
    current.setValue(percent);
    if ((csize != 0) && (tsize != 0))
    {
      int done = csize / 1024;
      int remain = tsize / 1024;
      if ((done / 1024 >= 1) && (remain / 1024 >= 1))
      {
        int indx = 3;
        String value1 = String.valueOf((float) done / 1024);
        String value2 = String.valueOf((float) remain / 1024);
        if ((value1.length() < 4) || (value2.length() < 4)) indx = (value1.length() < value2.length() ? value1.length() : value2.length()) - 1;
        lStatus.setText(value1.substring(0, value1.indexOf(".") + indx) + " / " + value2.substring(0, value2.indexOf(".") + indx) + " Mo");
      } else
        lStatus.setText(String.valueOf(done) + "/" + String.valueOf(remain) + " Ko");
    }
    this.setTitle(String.valueOf(percent) + "% - " + fileName + " - WebHunter");
  }

  // ---- Display a string in the status bar ----

  public void setProgressString(String msg)
  {
    lStatus.setText(msg);
  }

  // ---- Clear the URL textfield

  public void setUrlVoid()
  {
    tUrl.setText("");
  }

  /*
   * Instances management
   */

  // ---- Create a new window ----

  public static WebHunter addInstance(boolean showGUI)
  {
    WebHunter myGUI = new WebHunter("Web Hunter");
    usrProps = System.getProperty("user.home") + File.separator + ".webhunter.props";
    if (usrProps != null)
    {
      try
      {
        loadProps(null, USER_PROPS);
      } catch (FileNotFoundException fnf) { }
      catch (IOException e)
      {
        System.err.println("Error while loading user"  + " properties:");
        e.printStackTrace();
      }
      loadProperties();
    }
    instances.addElement(myGUI);
    if (showGUI) myGUI.show();
    return myGUI;
  }

  // ---- Destroy current window ----

  public void removeInstance()
  {
    instances.removeElement(this);
    this.dispose();
  }

  // ---- Return the amount of windows ----

  public int getElements()
  {
    return instances.size();
  }

  /*
   * Show an error message
   */

  public void showMessage(String msg, String title)
  {
    JOptionPane.showMessageDialog(this, msg, title, JOptionPane.WARNING_MESSAGE);
  }

  /*
   * Main
   */

  public static void main(String args[])
  {
    SplashScreen splash = new SplashScreen();
    WebHunter myGUI = addInstance(false);
    splash.dispose();
    myGUI.show();
  }
}

// End of WebHunter.java
